'use strict';

const upath = require('upath');
const ufo = require('ufo');
const fsExtra = require('fs-extra');
const crypto = require('crypto');
const MagicString = require('magic-string');
const estreeWalker = require('estree-walker');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

const crypto__default = /*#__PURE__*/_interopDefaultLegacy(crypto);
const MagicString__default = /*#__PURE__*/_interopDefaultLegacy(MagicString);

var name = "@nuxtjs/composition-api";
var version = "0.21.0-vite-1";

function createKey(source, method = "base64") {
  const hash = crypto__default['default'].createHash("md5");
  hash.update(source);
  return hash.digest(method).toString();
}
function compositionApiPlugin() {
  return {
    name: "nuxt:composition-api",
    enforce: "pre",
    transform(code, filename) {
      try {
        const {0: script = code, index: codeIndex = 0} = code.match(/(?<=<script[^>]*>)[\S\s.]*?(?=<\/script>)/) || [];
        const ast = this.parse(script);
        const s = new MagicString__default['default'](code);
        estreeWalker.walk(ast, {
          enter(node) {
            var _a, _b;
            const {start, end} = node;
            const {value, callee, arguments: args = []} = node;
            if (node.type === "Literal" && value === "@nuxtjs/composition-api") {
              s.overwrite(codeIndex + start, codeIndex + end, "'~@nuxtjs/composition-api/vite'");
            }
            if ((callee == null ? void 0 : callee.type) === "Identifier" || ((_a = callee == null ? void 0 : callee.property) == null ? void 0 : _a.type) === "Identifier") {
              let method = "base64";
              switch (callee.name || ((_b = callee.property) == null ? void 0 : _b.name)) {
                case "useStatic":
                  if (args.length > 2)
                    return;
                  if (args.length === 2) {
                    s.prependLeft(codeIndex + end - 1, ", undefined");
                  }
                  method = "hex";
                  break;
                case "shallowSsrRef":
                case "ssrPromise":
                case "ssrRef":
                case "reqSsrRef":
                case "useAsync":
                  if (args.length > 1)
                    return;
                  break;
                default:
                  return;
              }
              s.appendLeft(codeIndex + end - 1, ", '" + createKey(`${filename}-${end}`, method) + "'");
            }
          }
        });
        return {
          code: s.toString(),
          map: s.generateMap().toString()
        };
      } catch (e) {
      }
    }
  };
}

function isFullStatic(options) {
  var _a;
  return !options.dev && !options._legacyGenerate && options.target === "static" && ((_a = options.render) == null ? void 0 : _a.ssr);
}
function isUrl(url) {
  return ["http", "//"].some((str) => url.startsWith(str));
}
const compositionApiModule = function compositionApiModule2() {
  var _a, _b, _c, _d;
  let corejsPolyfill = this.nuxt.options.build.corejs ? String(this.nuxt.options.build.corejs) : void 0;
  try {
    if (!["2", "3"].includes(corejsPolyfill || "")) {
      const corejsPkg = this.nuxt.resolver.requireModule("core-js/package.json");
      corejsPolyfill = corejsPkg.version.slice(0, 1);
    }
  } catch (e) {
    corejsPolyfill = void 0;
  }
  const libRoot = upath.resolve(__dirname, "..");
  const {dst: pluginDst} = this.addTemplate({
    src: upath.resolve(libRoot, "templates", "plugin.js"),
    fileName: upath.join("composition-api", "plugin.js")
  });
  this.addPlugin({
    src: upath.resolve(libRoot, "templates", "polyfill.client.js"),
    fileName: upath.join("composition-api", "polyfill.client.js"),
    options: {
      corejsPolyfill
    }
  });
  const {dst: metaPluginDst} = this.addTemplate({
    src: upath.resolve(libRoot, "templates", "meta.js"),
    fileName: upath.join("composition-api", "meta.js")
  });
  const staticPath = upath.join(this.options.buildDir || "", "static-json");
  this.nuxt.hook("builder:prepared", () => {
    fsExtra.mkdirpSync(staticPath);
  });
  this.nuxt.hook("generate:route", () => {
    fsExtra.mkdirpSync(staticPath);
  });
  this.nuxt.hook("generate:done", async (generator) => {
    if (!fsExtra.existsSync(staticPath))
      return;
    const {distPath} = generator;
    fsExtra.readdirSync(staticPath).forEach((file) => fsExtra.copyFileSync(upath.join(staticPath, file), upath.join(distPath, file)));
  });
  const globalName = this.options.globalName || "nuxt";
  const globalContextFactory = ((_a = this.options.globals) == null ? void 0 : _a.context) || ((globalName2) => `__${globalName2.toUpperCase()}__`);
  const globalNuxtFactory = ((_b = this.options.globals) == null ? void 0 : _b.nuxt) || ((globalName2) => `$${globalName2}`);
  const globalContext = globalContextFactory(globalName);
  const globalNuxt = globalNuxtFactory(globalName);
  const routerBase = ufo.withTrailingSlash((_c = this.options.router) == null ? void 0 : _c.base);
  const publicPath = ufo.withTrailingSlash((_d = this.options.build) == null ? void 0 : _d.publicPath);
  const {dst: entryDst} = this.addTemplate({
    src: upath.resolve(libRoot, "lib", "entrypoint.es.js"),
    fileName: upath.join("composition-api", "index.js"),
    options: {
      isFullStatic: isFullStatic(this.nuxt.options),
      staticPath,
      publicPath: isUrl(publicPath) ? publicPath : routerBase,
      globalContext,
      globalNuxt
    }
  });
  const entryFile = upath.resolve(this.options.buildDir || "", entryDst);
  this.options.build.transpile = this.options.build.transpile || [];
  this.options.build.transpile.push(/@nuxtjs[\\/]composition-api/, entryFile);
  const aliases = ["@nuxtjs/composition-api", "~@nuxtjs/composition-api/vite"];
  for (const alias of aliases) {
    this.options.alias[alias] = entryFile;
  }
  this.nuxt.hook("vite:extend", (ctx) => {
    ctx.config.plugins.push(compositionApiPlugin());
    ctx.config.optimizeDeps.exclude.push("@vue/composition-api");
  });
  this.options.build = this.options.build || {};
  this.options.build.babel = this.options.build.babel || {};
  this.options.build.babel.plugins = this.options.build.babel.plugins || [];
  if (this.options.build.babel.plugins instanceof Function) {
    console.warn("Unable to automatically add Babel plugin. Make sure your custom `build.babel.plugins` returns `@nuxtjs/composition-api/babel`");
  } else {
    this.options.build.babel.plugins.push(upath.join(__dirname, "babel"));
  }
  const actualPresets = this.options.build.babel.presets;
  this.options.build.babel.presets = (env, [defaultPreset, defaultOptions]) => {
    const newOptions = {
      ...defaultOptions,
      jsx: {
        ...typeof defaultOptions.jsx === "object" ? defaultOptions.jsx : {},
        compositionAPI: true
      }
    };
    if (typeof actualPresets === "function") {
      return actualPresets(env, [defaultPreset, newOptions]);
    }
    return [[defaultPreset, newOptions]];
  };
  this.options.plugins = this.options.plugins || [];
  this.options.plugins.unshift(upath.resolve(this.options.buildDir || "", pluginDst));
  if (!(this.nuxt.options.buildModules || []).includes("@nuxtjs/pwa") && !this.nuxt.options.modules.includes("@nuxtjs/pwa")) {
    this.options.plugins.push(upath.resolve(this.options.buildDir || "", metaPluginDst));
  } else if (this.nuxt.options.dev) {
    console.warn("useMeta is not supported in onGlobalSetup as @nuxtjs/pwa detected.\nSee https://github.com/nuxt-community/composition-api/issues/307");
  }
};
compositionApiModule.meta = {
  name,
  version
};
const warnToAddModule = () => {
  console.error("You need to add `@nuxtjs/composition-api` to your buildModules in order to use it. See https://composition-api.nuxtjs.org/getting-started/setup.");
  throw new Error("You need to add `@nuxtjs/composition-api` to your buildModules in order to use it. See https://composition-api.nuxtjs.org/getting-started/setup.");
};
const helperFunctions = JSON.parse(`["computed","createApp","createRef","customRef","defineAsyncComponent","del","getCurrentInstance","h","inject","isRaw","isReactive","isReadonly","isRef","markRaw","nextTick","onActivated","onBeforeMount","onBeforeUnmount","onBeforeUpdate","onDeactivated","onErrorCaptured","onMounted","onServerPrefetch","onUnmounted","onUpdated","provide","proxyRefs","reactive","readonly","ref","set","shallowReactive","shallowReadonly","shallowRef","toRaw","toRef","toRefs","triggerRef","unref","useCssModule","useCSSModule","version","warn","watch","watchEffect","useAsync","defineComponent","useContext","withContext","useFetch","globalPlugin","onGlobalSetup","setMetaPlugin","useMeta","reqRef","reqSsrRef","ssrRef","shallowSsrRef","setSSRContext","ssrPromise","useStatic","defineNuxtPlugin","defineNuxtMiddleware","defineNuxtModule","defineNuxtServerMiddleware","defineNuxtConfig","wrapProperty","useRouter","useRoute","useStore"]`);
helperFunctions.forEach((helper) => {
  compositionApiModule[helper] = warnToAddModule;
});
compositionApiModule.defineNuxtConfig = (config) => config;

module.exports = compositionApiModule;
